@@ -48,21 +48,25 @@ import tempfile
 import sys
 import os
 import subprocess
-from .transcript import gtf_to_cds
-from .transcript import cds_to_tree
+from .transcript import gtf_to_cds, cds_to_feature_length, cds_to_tree
 from distutils.core import Command
 
 download = {
-    "GENCODE v29 annotation": [
+    "GENCODE v34 annotation": [
         "ftp://ftp.ebi.ac.uk/pub/databases/gencode/"
-        "Gencode_human/release_29/"
-        "gencode.v29.annotation.gtf.gz"
+        "Gencode_human/release_34/"
+        "gencode.v34.annotation.gtf.gz"
     ],
     "GENCODE v19 annotation": [
         "ftp://ftp.ebi.ac.uk/pub/databases/gencode/"
         "Gencode_human/release_19/"
         "gencode.v19.annotation.gtf.gz"
     ],
+    "GENCODE vM25 annotation": [
+        "ftp://ftp.ebi.ac.uk/pub/databases/gencode/"
+        "Gencode_mouse/release_M25/"
+        "gencode.vM25.annotation.gtf.gz"
+    ],
     "GENCODE vM1 annotation": [
         "ftp://ftp.ebi.ac.uk/pub/databases/gencode/"
         "Gencode_mouse/release_M1/"
@@ -76,6 +80,9 @@ download = {
         "ftp://ftp.ccb.jhu.edu/pub/data/bowtie_indexes/hg19.ebwt.zip",
         "http://verve.webfactional.com/hg19.ebwt.zip",
     ],
+    "Bowtie UCSC mm10 index": [
+        "http://verve.webfactional.com/misc/mm10_bowtie_index.zip",
+    ],
     "Bowtie UCSC mm9 index": [
         "ftp://ftp.ccb.jhu.edu/pub/data/bowtie_indexes/mm9.ebwt.zip",
         "http://verve.webfactional.com/mm9.ebwt.zip",
@@ -458,7 +465,7 @@ class NeoepiscopeDownloader(object):
             else:
                 self.curl_exe = self.curl_exe.replace(" --help", "")
         self._print_to_screen_and_log("[Configuring] Downloading mhcflurry data...")
-        subprocess.call(["mhcflurry-downloads", "fetch"])
+        subprocess.call(["mhcflurry-downloads", "fetch", "models_class1_presentation"])
         if self.download_dir is None:
             self.download_dir = os.path.join(
                 os.path.expanduser("~"), "neoepiscope.data"
@@ -513,20 +520,20 @@ class NeoepiscopeDownloader(object):
             os.rmdir(self.download_dir)
             pass
         os.chdir(temp_install_dir)
-        if self._yes_no_query("Download GENCODE v29 gtf annotation file?"):
+        if self._yes_no_query("Download GENCODE v34 gtf annotation file?"):
             self._grab_and_explode(
-                download["GENCODE v29 annotation"],
-                "GENCODE v29 annotation",
+                download["GENCODE v34 annotation"],
+                "GENCODE v34 annotation",
                 explode=False,
             )
-            gencode_v29_temp = os.path.join(temp_install_dir, "gencode_v29")
-            gencode_v29 = os.path.join(self.download_dir, "gencode_v29")
-            gencode_v29_gtf = os.path.join(
+            gencode_v34_temp = os.path.join(temp_install_dir, "gencode_v34")
+            gencode_v34 = os.path.join(self.download_dir, "gencode_v34")
+            gencode_v34_gtf = os.path.join(
                 temp_install_dir,
-                os.path.basename(download["GENCODE v29 annotation"][0]),
+                os.path.basename(download["GENCODE v34 annotation"][0]),
             )
             try:
-                os.makedirs(gencode_v29_temp)
+                os.makedirs(gencode_v34_temp)
             except OSError as e:
                 self._print_to_screen_and_log(
                     (
@@ -534,14 +541,15 @@ class NeoepiscopeDownloader(object):
                         "directory %s for installation. May need "
                         "sudo permissions."
                     )
-                    % gencode_v29_temp
+                    % gencode_v34_temp
                 )
                 self._bail()
-            self._print_to_screen_and_log("[Configuring] Indexing GENCODE v29...")
-            cds_dict, tx_data_dict = gtf_to_cds(gencode_v29_gtf, gencode_v29_temp)
-            cds_to_tree(cds_dict, gencode_v29_temp)
+            self._print_to_screen_and_log("[Configuring] Indexing GENCODE v34...")
+            cds_dict, tx_data_dict = gtf_to_cds(gencode_v34_gtf, gencode_v34_temp)
+            feature_lengths = cds_to_feature_length(cds_dict, tx_data_dict, gencode_v34_temp)
+            cds_to_tree(cds_dict, gencode_v34_temp)
         else:
-            gencode_v29 = None
+            gencode_v34 = None
         if self._yes_no_query("Download GENCODE v19 gtf annotation file?"):
             self._grab_and_explode(
                 download["GENCODE v19 annotation"],
@@ -568,9 +576,40 @@ class NeoepiscopeDownloader(object):
                 self._bail()
             self._print_to_screen_and_log("[Configuring] Indexing GENCODE v19...")
             cds_dict, tx_data_dict = gtf_to_cds(gencode_v19_gtf, gencode_v19_temp)
+            feature_lengths = cds_to_feature_length(cds_dict, tx_data_dict, gencode_v19_temp)
             cds_to_tree(cds_dict, gencode_v19_temp)
         else:
             gencode_v19 = None
+        if self._yes_no_query("Download GENCODE vM25 gtf annotation file?"):
+            self._grab_and_explode(
+                download["GENCODE vM25 annotation"],
+                "GENCODE vM25 annotation",
+                explode=False,
+            )
+            gencode_vM25_temp = os.path.join(temp_install_dir, "gencode_vM25")
+            gencode_vM25 = os.path.join(self.download_dir, "gencode_vM25")
+            gencode_vM25_gtf = os.path.join(
+                temp_install_dir,
+                os.path.basename(download["GENCODE vM25 annotation"][0]),
+            )
+            try:
+                os.makedirs(gencode_vM25_temp)
+            except OSError as e:
+                self._print_to_screen_and_log(
+                    (
+                        "Problem encountered trying to create "
+                        "directory %s for installation. May need "
+                        "sudo permissions."
+                    )
+                    % gencode_vM25_temp
+                )
+                self._bail()
+            self._print_to_screen_and_log("[Configuring] Indexing GENCODE vM25...")
+            cds_dict, tx_data_dict = gtf_to_cds(gencode_vM25_gtf, gencode_vM25_temp)
+            feature_lengths = cds_to_feature_length(cds_dict, tx_data_dict, gencode_vM25_temp)
+            cds_to_tree(cds_dict, gencode_vM25_temp)
+        else:
+            gencode_vM25 = None
         if self._yes_no_query("Download GENCODE vM1 gtf annotation file?"):
             self._grab_and_explode(
                 download["GENCODE vM1 annotation"],
@@ -597,6 +636,7 @@ class NeoepiscopeDownloader(object):
                 self._bail()
             self._print_to_screen_and_log("[Configuring] Indexing GENCODE vM1...")
             cds_dict, tx_data_dict = gtf_to_cds(gencode_vM1_gtf, gencode_vM1_temp)
+            feature_lengths = cds_to_feature_length(cds_dict, tx_data_dict, gencode_vM1_temp)
             cds_to_tree(cds_dict, gencode_vM1_temp)
         else:
             gencode_vM1 = None
@@ -616,6 +656,13 @@ class NeoepiscopeDownloader(object):
             bowtie_hg19 = os.path.join(self.download_dir, "hg19")
         else:
             bowtie_hg19 = None
+        if self._yes_no_query("Download Bowtie UCSC mm10 index?"):
+            self._grab_and_explode(
+                download["Bowtie UCSC mm10 index"], "Bowtie UCSC mm10 index"
+            )
+            bowtie_mm10 = os.path.join(self.download_dir, "mm10")
+        else:
+            bowtie_mm10 = None
         if self._yes_no_query("Download Bowtie UCSC mm9 index?"):
             self._grab_and_explode(
                 download["Bowtie UCSC mm9 index"], "Bowtie UCSC mm9 index"
@@ -624,8 +671,9 @@ class NeoepiscopeDownloader(object):
         else:
             bowtie_mm9 = None
         programs = []
-        for program in ["netMHCIIpan v3", "netMHCpan v3", "netMHCpan v4", 
-                        "netMHC v4", "netMHCII v2", "PickPocket v1", "netMHCstabpan v1"]:
+        for program in ["netMHCIIpan v3", "netMHCIIpan v4", "netMHCpan v3", "netMHCpan v4.0", 
+                        "netMHCpan v4.1", "netMHC v4", "netMHCII v2", "PickPocket v1", 
+                        "netMHCstabpan v1"]:
             if self._yes_no_query(
                 (
                     "Do you have an install of {} "
@@ -711,15 +759,19 @@ Defines default paths of neoepiscope's dependencies
 None indicates the user didn't install the tool or data
 \"""
 
-gencode_v29 = {gencode_v29}
+gencode_v34 = {gencode_v34}
 gencode_v19 = {gencode_v19}
+gencode_vM25 = {gencode_vM25}
 gencode_vM1 = {gencode_vM1}
 bowtie_grch38 = {bowtie_grch38}
 bowtie_hg19 = {bowtie_hg19}
+bowtie_mm10 = {bowtie_mm10}
 bowtie_mm9 = {bowtie_mm9}
 netMHCIIpan3 = {netMHCIIpan3}
+netMHCIIpan4 = {netMHCIIpan4}
 netMHCpan3 = {netMHCpan3}
 netMHCpan4 = {netMHCpan4}
+netMHCpan4_1 = {netMHCpan4_1}
 netMHC4 = {netMHC4}
 netMHCII2 = {netMHCII2}
 PickPocket1 = {PickPocket1}
@@ -729,12 +781,15 @@ hapcut2_hairs = {hapcut2_hairs}
 hapcut2 = {hapcut2}
 """
                 ).format(
-                    gencode_v29=(
-                        "None" if gencode_v29 is None else self._quote(gencode_v29)
+                    gencode_v34=(
+                        "None" if gencode_v34 is None else self._quote(gencode_v34)
                     ),
                     gencode_v19=(
                         "None" if gencode_v19 is None else self._quote(gencode_v19)
                     ),
+                    gencode_vM25=(
+                        "None" if gencode_vM1 is None else self._quote(gencode_vM25)
+                    ),
                     gencode_vM1=(
                         "None" if gencode_vM1 is None else self._quote(gencode_vM1)
                     ),
@@ -744,39 +799,48 @@ hapcut2 = {hapcut2}
                     bowtie_hg19=(
                         "None" if bowtie_hg19 is None else self._quote(bowtie_hg19)
                     ),
+                    bowtie_mm10=(
+                        "None" if bowtie_mm10 is None else self._quote(bowtie_mm10)
+                    ),
                     bowtie_mm9=(
                         "None" if bowtie_mm9 is None else self._quote(bowtie_mm9)
                     ),
                     netMHCIIpan3=(
                         "None" if programs[0] is None else self._quote(programs[0])
                     ),
-                    netMHCpan3=(
+                    netMHCIIpan4=(
                         "None" if programs[1] is None else self._quote(programs[1])
                     ),
-                    netMHCpan4=(
+                    netMHCpan3=(
                         "None" if programs[2] is None else self._quote(programs[2])
                     ),
-                    netMHC4=(
+                    netMHCpan4=(
                         "None" if programs[3] is None else self._quote(programs[3])
                     ),
-                    netMHCII2=(
+                    netMHCpan4_1=(
                         "None" if programs[4] is None else self._quote(programs[4])
                     ),
-                    PickPocket1=(
+                    netMHC4=(
                         "None" if programs[5] is None else self._quote(programs[5])
                     ),
-                    netMHCstabpan1=(
+                    netMHCII2=(
                         "None" if programs[6] is None else self._quote(programs[6])
                     ),
-                    PSSMHCpan1=(
+                    PickPocket1=(
                         "None" if programs[7] is None else self._quote(programs[7])
                     ),
-                    hapcut2_hairs=(
+                    netMHCstabpan1=(
                         "None" if programs[8] is None else self._quote(programs[8])
                     ),
-                    hapcut2=(
+                    PSSMHCpan1=(
                         "None" if programs[9] is None else self._quote(programs[9])
                     ),
+                    hapcut2_hairs=(
+                        "None" if programs[10] is None else self._quote(programs[10])
+                    ),
+                    hapcut2=(
+                        "None" if programs[11] is None else self._quote(programs[11])
+                    ),
                 ),
                 file=paths_stream,
             )
