@@ -582,7 +582,9 @@ def get_vaf_pos(VCF):
     return (vaf_pos, field)
 
 
-def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
+def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict, 
+                  tpm_dict=None, tpm_threshold=None, expressed_variants=None,
+                  covered_variants=None):
     """ Writes predicted neoepitopes out to file
 
         output_file: path to output file
@@ -591,6 +593,12 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
         tool_dict: dictionary storing prediction tool data
         tx_dict: dictionary linking transcript ID to list of 
                     [transcript type, gene ID, gene name]
+        tpm_dict: dictionary linking feature ID to TPM value
+        tmp_threshold: minimum TPM to retain peptide
+        expressed_variants: dictionary linking variants to count of RNA-seq 
+                            reads supporting them
+        covered_variants: dictionary linking variants to count of RNA-seq 
+                          reads covering their position
 
         Return value: None.
     """
@@ -632,6 +640,10 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
             "Transcript_type",
             "Gene_ID",
             "Gene_name",
+            "TPM",
+            "Variant_read_support",
+            "Variant_read_coverage",
+            "Percent_read_support",
             "IEDB_ID"
         ]
         for allele in hla_alleles:
@@ -670,6 +682,34 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
                     vaf = str(mutation[5])
                 # Get transcript/gene info
                 tx_info = tx_dict[mutation[8]]
+                # Get gene expression info
+                if tpm_dict is not None:
+                    try:
+                        # Search transcript TPM first
+                        tpm = tpm_dict[mutation[8]]
+                    except KeyError:
+                        tpm = 'NA'
+                    if tpm_threshold is not None:
+                        if tpm == 'NA':
+                            continue
+                        elif tpm < tpm_threshold:
+                            continue
+                else:
+                    tpm = "NA"
+                if expressed_variants is not None:
+                    reads_supporting_variant = expressed_variants[tuple(mutation[0:5])]
+                    reads_covering_variant = covered_variants[tuple(mutation[0:5])]
+                    try:
+                        percent_support = round(
+                            100*float(reads_supporting_variant)/float(reads_covering_variant),
+                            3
+                        )
+                    except ZeroDivisionError:
+                        percent_support = 'NA'
+                else:
+                    reads_supporting_variant = 'NA'
+                    reads_covering_variant = 'NA'
+                    percent_support = 'NA'
                 out_line = [
                     epitope,
                     mutation[0],
@@ -684,6 +724,10 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
                     tx_info[0],
                     tx_info[1],
                     tx_info[2],
+                    str(tpm),
+                    str(reads_supporting_variant),
+                    str(reads_covering_variant),
+                    str(percent_support),
                     iedb_id
                 ]
                 for i in range(9, len(mutation)):
@@ -710,8 +754,23 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
                         vaf = "NA"
                     else:
                         vaf = str(mut[5])
+                    if expressed_variants is not None:
+                        reads_supporting_variant = expressed_variants[tuple(mut[0:5])]
+                        reads_covering_variant = covered_variants[tuple(mut[0:5])]
+                        try:
+                            percent_support = round(
+                                100*float(reads_supporting_variant)/float(reads_covering_variant),
+                                3
+                            )
+                        except ZeroDivisionError:
+                            percent_support = 'NA'
+                    else:
+                        reads_supporting_variant = 'NA'
+                        reads_covering_variant = 'NA'
+                        percent_support = 'NA'
                     mutation_dict[
-                        (mut[0], mut[1], ref, alt, mut[4], vaf, mut[6])
+                        (mut[0], mut[1], ref, alt, mut[4], vaf, mut[6], 
+                         reads_supporting_variant, reads_covering_variant, percent_support)
                     ].append([mut[7], mut[8]])
                 mutation_list = sorted(list(mutation_dict.keys()))
                 # Get transcript/gene info 
@@ -720,10 +779,29 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
                     tx_types = []
                     gene_ids = []
                     gene_names = []
+                    tpm_values = []
+                    expressed = False
+                    # Get transcript/gene info
                     for tx in transcripts:
                         tx_types.append(tx_dict[tx][0])
                         gene_ids.append(tx_dict[tx][1])
                         gene_names.append(tx_dict[tx][2])
+                        # Get gene expression info
+                        if tpm_dict is not None:
+                            try:
+                                # Try transcript TPM first
+                                tpm = tpm_dict[tx]
+                                if tpm_threshold is not None:
+                                    if tpm >= tpm_threshold:
+                                        expressed = True
+                            except KeyError:
+                                tpm = 'NA'
+                            tpm_values.append(tpm)
+                        else:
+                            tpm_values.append('NA')
+                    # If no gene for mutation is expressed, filter out
+                    if tpm_threshold is not None and not expressed:
+                        continue
                     out_line = [
                         epitope,
                         mut[0],
@@ -738,6 +816,10 @@ def write_results(output_file, hla_alleles, neoepitopes, tool_dict, tx_dict):
                         ";".join(tx_types),
                         ";".join(gene_ids),
                         ";".join(gene_names),
+                        ";".join([str(x) for x in tpm_values]),
+                        str(mut[7]),
+                        str(mut[8]),
+                        str(mut[9]),
                         iedb_id
                     ]
                     for score in ep_scores:
